package com.supermap.sample;

import java.util.ArrayList;
import java.util.List;
import com.supermap.services.components.*;
import com.supermap.services.components.commontypes.*;
import com.supermap.services.components.impl.MapImpl;
import com.supermap.services.components.spi.MapProvider;
import com.supermap.services.providers.RestMapProvider;
import com.supermap.services.providers.RestMapProviderSetting;

public class MapComponentSampleUseRMP {

    /**
     * @param args
     * @throws MapException
     */
    public static void main(String[] args) throws MapException {
        MapImpl mapComponent = createMapComponent();
        // SQL查询
        // SQL Query
        queryBySQLSample(mapComponent);
        // 根据显示范围出图
        // Output the images according to the display range
        getMapImageSample(mapComponent);
        mapComponent.dispose();
    }

    public static MapImpl createMapComponent() {
        // 初始化一个REST地图服务提供者。
        // Initialize a REST map provider.
        RestMapProviderSetting providerSetting = new RestMapProviderSetting();
        //指定REST地图服务提供者所使用的REST服务地址。
        //Specify the REST service address used by REST provider.
        providerSetting.restServiceRootURL = "http://localhost:8090/iserver/services/map-china400/rest";
        // 构建 RestMapProvider 对象
        // Construct the RestMapProvider object
        RestMapProvider restMapProvider = new RestMapProvider(providerSetting);
        List<MapProvider> providers = new ArrayList<MapProvider>();
        providers.add(restMapProvider);

        // 初始化地图服务组件上下文对象。
        // Initialize map component context object.
        MapContext mapContext = new MapContext();
        mapContext.setProviders(providers);
        // 创建地图服务组件。
        // Create map service component.
        MapImpl mapComponent = new MapImpl();
        //指定地图服务组件使用的服务提供者
        //Specify the service provider used by map component.
        mapComponent.setComponentContext(mapContext);
        return mapComponent;

    }

    /**
     * <p>
     * 查询示范
     * </p>
     * <p>
     * Query samples
     * </p>
     * @param mapComponent
     * @throws MapException
     */
    public static void queryBySQLSample(MapImpl mapComponent) throws MapException {
        // 属性查询参数集合。
        // Attribute query parameter set.
        QueryParameterSet queryParameters = new QueryParameterSet();
        QueryParameter[] queryLayerParams = new QueryParameter[1];
        queryLayerParams[0] = new QueryParameter();
        queryLayerParams[0].name = "China_Capital_P@China400";
        // 属性过滤条件
        // Attributes filter
        queryLayerParams[0].attributeFilter = "SMID<6";
        // 返回的属性字段名称（不区分大小写）。
        // The name of return attribute field (case-insensitive ).
        queryLayerParams[0].fields = new String[] { "AdminName", "AdminCode" };
        queryParameters.queryParams = queryLayerParams;
        // 设置查询结果包含属性信息及空间信息。
        // Set the query result to contain attribute and spatial information.
        queryParameters.queryOption = QueryOption.ATTRIBUTEANDGEOMETRY;

        // 查询在“China”地图的“China_Capital_P@China400”图层中 SmID小于6 所有的要素。
        // Query all the festures that the SmID is less than 6 on "China_Capital_P@China400" layer in China.
        QueryResult queryResult = mapComponent.queryBySQL("China", queryParameters);
        OutputQueryResult(queryResult);
    }

    /**
     * <p>
     * 出图示范
     * </p>
     * <p>
     * Output samples
     * </p>
     * @param mapComponent
     * @throws MapException
     */
    public static void getMapImageSample(MapImpl mapComponent) throws MapException {
        // 显示范围。
        // Display the range.
        Rectangle2D viewBounds = new Rectangle2D(9871111.11 ,2371111.11 , 12128888.89 , 4628888.89);
        // 获取“China”默认的地图参数。
        // Get the default map parameter of "China".
        MapParameter mapParameter = mapComponent.getDefaultMapParameter("China");
        // 图片输出设置，指定了图片输出格式、返回图片地址、出图范围，以及使用默认图片大小（256*256）出图。
        // The image outputting setting specifies the format, return image address, range and the size of default image (256*256).
        ImageOutputOption outputOption = new ImageOutputOption();
        mapParameter.returnType = ReturnType.URL;
        outputOption.format = OutputFormat.PNG;
        mapParameter.viewBounds = viewBounds;
        MapImage mapImage = mapComponent.getMapImage(mapParameter, outputOption);
        System.out.println("The path of the output image is: " + mapImage.imageUrl);
    }

    /**
     * <p>
     * 获取要素字段值
     * </p>
     * <p>
     * Get the feature field value
     * </p>
     * @param fieldName
     * @param feature
     * @return
     */
    public static String getfieldValueByName(String fieldName, Feature feature) {
        for (int i = 0; i < feature.fieldNames.length; i++) {
            if (feature.fieldNames[i].equalsIgnoreCase(fieldName)) {
                return feature.fieldValues[i];
            }
        }
        return "error";
    }

    /**
     * <p>
     * 输出查询结果
     * </p>
     * <p>
     * Output the query result
     * </p>
     * @param fieldName
     * @param feature
     * @return
     */
    public static void OutputQueryResult(QueryResult queryResult) {
        System.out.println("\n The query result is as follows: \n--------------------------------------------");
        System.out.println("The total count of features is" + queryResult.totalCount);
        System.out.println("The current count is" + queryResult.currentCount);

        Recordset recordset[] = queryResult.recordsets;
        System.out.println("There are "+recordset.length+" query record sets in the query result: ");

        for (int i = 0; i < recordset.length; i++) {
            System.out.println("The dataset corresponding to query record set " + ( i + 1 ) + "is: " + recordset[i].datasetName + ". The Country attributes of the result features are listed as follows:");
            Feature features[] = recordset[i].getFeatures();
            for (int j = 0; j < features.length; j++) {
                System.out.println((j + 1) + ") " + getfieldValueByName("AdminName", features[j]));
            }
        }
    }
}
